#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "libusb.h"

static const int CONTROL_REQUEST_TYPE_IN = LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE;
static const int CONTROL_REQUEST_TYPE_OUT = LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE;

static const int HID_GET_REPORT = 0x01;
static const int HID_SET_REPORT = 0x09;
static const int HID_REPORT_TYPE_INPUT = 0x01;
static const int HID_REPORT_TYPE_OUTPUT = 0x02;
static const int HID_REPORT_TYPE_FEATURE = 0x03;

static const int MAX_CONTROL_IN_TRANSFER_SIZE = 9;
static const int MAX_CONTROL_OUT_TRANSFER_SIZE = 9;

static const int INTERFACE_NUMBER = 0;
static const int TIMEOUT_MS = 5000;

static const int VENDOR_ID = 0x0483;
static const int PRODUCT_ID = 0x5750;

//Start byte "0x01" for transmit and "0x02" for receive is necessary, following 8 bytes are user data
//Overall length both, transmitted and received data should be 9 bytes
//Array from PC to MCU, you should write data here and keep first byte to "0x02" before sending
uint8_t data_out[9] = {0x02, 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H'};
//Array from MCU to PC, this will be overwriten every time you call data exchange function
uint8_t data_in[9];

int exchange_input_and_output_reports_via_interrupt_transfers(libusb_device_handle *devh){
	static const int INTERRUPT_IN_ENDPOINT = 0x81;
	static const int INTERRUPT_OUT_ENDPOINT = 0x01;
	static const int MAX_INTERRUPT_OUT_TRANSFER_SIZE = 9;
	int bytes_transferred;
	int i = 0;
	int result = 0;;
	result = libusb_interrupt_transfer(
			devh,
			INTERRUPT_OUT_ENDPOINT,
			data_out,
			MAX_INTERRUPT_OUT_TRANSFER_SIZE,
			&bytes_transferred,
			TIMEOUT_MS);
	if (result >= 0){
	  	printf("Data sent via interrupt transfer:\n");
	  	for(i = 0; i < bytes_transferred; i++){
	  		printf("%02x ",data_out[i]);
	  	}
	  	printf("\n");
		result = libusb_interrupt_transfer(
				devh,
				INTERRUPT_IN_ENDPOINT,
				data_in,
				MAX_INTERRUPT_OUT_TRANSFER_SIZE,
				&bytes_transferred,
				TIMEOUT_MS);
		if (result >= 0){
			if (bytes_transferred > 0){
			  	printf("Data received via interrupt transfer:\n");
			  	for(i = 0; i < bytes_transferred; i++)
			  	{
			  		printf("%02x ",data_in[i]);
			  	}
			  	printf("\n");
			}
			else{
				fprintf(stderr, "No data received in interrupt transfer (%d)\n", result);
				return -1;
			}
		}
		else{
			fprintf(stderr, "Error receiving data via interrupt transfer %d\n", result);
			return result;
		}
	}
	else{
		fprintf(stderr, "Error sending data via interrupt transfer %d\n", result);
		return result;
	}
  	return 0;
 }

int main(void){
	struct libusb_device_handle *devh = NULL;
	int device_ready = 0;
	int result;

	result = libusb_init(NULL);
	if (result >= 0)
	{
		devh = libusb_open_device_with_vid_pid(NULL, VENDOR_ID, PRODUCT_ID);

		if (devh != NULL)
		{
			result = libusb_claim_interface(devh, INTERFACE_NUMBER);
			if (result >= 0){
				device_ready = 1;
			}
			else{
				fprintf(stderr, "libusb_claim_interface error %d\n", result);
			}
		}
		else{
			fprintf(stderr, "Unable to find the device.\n");
		}
	}
	else{
		fprintf(stderr, "Unable to initialize libusb.\n");
	}
	if (device_ready){

		//This command will send "data_out" array to MCU and write received 9 bytes in "data_in"
		exchange_input_and_output_reports_via_interrupt_transfers(devh);
		//Here you can use received data from "data_in" array

		libusb_release_interface(devh, 0);
	}
	libusb_close(devh);
	libusb_exit(NULL);
	return 0;
}